##############################################################################
#
# MODULE:       r.randomwalk.cdf.r
#
# AUTHOR:       Martin Mergili
#
# PURPOSE:      Conceptual tool for mass movement propagation
#               Script for computing the cdf of the average angle of the path
#
# COPYRIGHT:    (c) 2013 - 2015 by the author
#               (c) 2013 - 2015 by the BOKU University, Vienna
#               (c) 2015        by the University of Vienna
#               (C) 1993 - 2015 by the R Development Core Team
#
# VERSION:      20151118 (18 November 2015)
#
#               This program is free software under the GNU General Public
#               License (>=v2).
#
##############################################################################

#Importing arguments (defined in r.randomwalk.py)
args<-commandArgs(trailingOnly = TRUE)
temppath<-args[1]
prefix<-args[2]
filepath<-args[3]
backfile<-args[4]
cdfmode<-as.integer(args[5])
imax<-as.integer(args[6])
retain<-as.numeric(args[7])
functype<-as.integer(args[8])

#Loading input data
if ( cdfmode==1 ) {
  gentable<-paste(filepath,prefix,'_backfile.txt', sep='')
} else {
  gentable<-backfile
}
gendata<-read.table(gentable, header=TRUE)

if ( nrow(gendata)>1 ) { #checking whether table contains more than one row

  #Suppressing scientific notation
  options(scipen=999)

  #Preprocessing data
  gendata$OMEGAT[gendata$OMEGAT<=0]<-NA
  gendata$OMEGAT<-tan(gendata$OMEGAT*0.0174532925)
  gendata$OMEGAT[gendata$OMEGAT>2]<-2

  #Creating plot file
  cdfplot=paste(prefix,'_results/',prefix,'_plots/',prefix,'_plot_cdf.png', sep='')
  png(filename=cdfplot, width=12, height=10, units='cm', res=300)

  #Defining margins
  par(mar=c(3.2,3.2,0.5,0.5))

  #Preparing random sampling of data
  if ( cdfmode==2 ) {
    maxcase=max(gendata$CASE)
    samplesize=as.integer(0.01*retain*maxcase)
    maxsample=maxcase
  }

  #Looping over all model runs
  sumx<-0
  sumix<-0
  for ( i in 1:imax ) {

    write('OMEGAT\tCDF', file=paste(temppath,'/cdfx',as.character(i),'.txt', sep=''), append=FALSE)

    #Sampling input data
    devdata<-gendata
    if ( cdfmode==2 ) {
      firstcase<-sample(1:maxsample,1)

      if ( firstcase <= maxsample-samplesize+1 ) {
        lastcase<-firstcase+samplesize-1
        devdata$OMEGAT[devdata$CASE>=firstcase & devdata$CASE<=lastcase]<-NA
      } else {
        lastcase<-firstcase+samplesize-maxsample-1
        devdata$OMEGAT[devdata$CASE>=firstcase | devdata$CASE<=lastcase]<-NA
      }
    }

    #Building pdf and cdf
    if ( functype==1 ) { #normal distribution
      mean=mean(devdata$OMEGAT, na.rm=TRUE)
      sd=sd(devdata$OMEGAT, na.rm=TRUE)
      x <- seq(0,2,length=2001)
      hx <- dnorm(x,mean,sd)
      ix <- pnorm(x,mean,sd)
    } else if ( functype==2 ) { #lognormal distribution
      mean=mean(log(devdata$OMEGAT), na.rm=TRUE)
      sd=sd(log(devdata$OMEGAT), na.rm=TRUE)
      x <- seq(0,2,length=2001)
      hx <- dlnorm(x,mean,sd)
      ix <- plnorm(x,mean,sd)
    }

    #updating sums (for computing average)
    sumx<-sumx+x
    sumix<-sumix+ix

    #Plotting histogram, pdf and cdf
    par(lwd=0.5)
    hist(devdata$OMEGAT, c(0.0,0.1,0.2,0.3,0.4,0.5,0.6,0.7,0.8,0.9,1.0,1.1,1.2,1.3,1.4,1.5,1.6,1.7,1.8,1.9,2.0), freq=FALSE, xlab=NULL, ylab=NULL, labels=NULL, axes=FALSE,
      xlim=c(0,1.5), ylim=c(-0.2*max(hx),1.2*max(hx)), main=NULL)
    par(lwd=1)
    par(new=TRUE)
    plot(x, hx, type="l", lty=5, lwd=1, col='brown', xlab=NULL, ylab=NULL, axes=FALSE, labels=NULL,
      xlim=c(0,1.5), ylim=c(-0.17*max(hx),1.02*max(hx)), main=NULL)
    par(new=TRUE)
    plot(x, ix, type="l", lty=1, lwd=1, col='blue', xlab=NULL, ylab=NULL, axes=FALSE,labels=NULL,
      xlim=c(0,1.5), ylim=c(-0.17,1.02), main=NULL)
    par(new=TRUE)

    #Writing cdf to temporary output file
    write.table(cbind(round(x,1),round(ix,6)), file=paste(temppath,'/cdfx',as.character(i),'.txt', sep=''), row.names=FALSE,
      col.names=FALSE, quote=FALSE, append=TRUE, sep='\t')

    #Writing sampling parameters to temporary output file
    if ( cdfmode == 2 ) {
      write(c(as.character(firstcase), as.character(lastcase)), file=paste(temppath,'/cdfthr',as.character(i),'.txt', sep=''), 
        append=FALSE, sep='\t')
    }
  }

  #Computing averages of pdf and cdf and writing to output file
  avgx<-sumx/imax
  avgix<-sumix/imax
  write('OMEGAT\tCDF', file=paste(filepath,prefix,'_cdf.txt', sep=''), append=FALSE)
  write.table(cbind(round(avgx,1),round(avgix,6)), file=paste(filepath,prefix,'_cdf.txt', sep=''), row.names=FALSE,
    col.names=FALSE, quote=FALSE, append=TRUE, sep='\t')

  #Plotting bounding box and axes
  box()
  axis(side=1, tck=-0.02, labels=NA)
  axis(side=2, tck=-0.02, labels=NA, at=c(0.0,0.2,0.4,0.6,0.8,1.0))
  axis(side=1, lwd=0, line=-0.4)
  axis(side=2, lwd=0, line=-0.4, labels=c('0.0','0.2','0.4','0.6','0.8','1.0'), at=c(0.0,0.2,0.4,0.6,0.8,1.0))

  #Plotting axis labels
  mtext(side=1, expression(Tangent~of~angle~of~reach~tan(italic(omega)[T])), line=1.9)
  mtext(side=2, 'Cumulative density', line=2)

  #Plotting legend
  legend('bottomleft', lty=c(5,1), lwd=c(1,1), col=c('brown','blue'), legend=c('Probability density','Cumulative density'),
    text.col='black', bty='n', horiz=TRUE, x.intersp=0.25, inset=c(0.01,0.01))

  #Closing plot file
  dev.off()
}
