##############################################################################
#
# MODULE:       r.randomwalk.roc.r
#
# AUTHOR:       Martin Mergili
#
# PURPOSE:      Conceptual tool for mass movement propagation
#               Script for ROC plot
#
# COPYRIGHT:    (c) 2013 - 2015 by the author
#               (c) 2013 - 2015 by the BOKU University, Vienna
#               (c) 2015        by the University of Vienna
#               (C) 1993 - 2015 by the R Development Core Team
#
# VERSION:      20151020 (20 October 2015)
#
#               This program is free software under the GNU General Public
#               License (>=v2).
#
##############################################################################

#Loading libraries
library(rgdal)
library(ROCR)

#Importing arguments (defined in r.randomwalk.py)
args<-commandArgs(trailingOnly = TRUE)
temppath<-args[1]
prefix<-args[2]
mode<-as.integer(args[3])
m<-as.integer(args[4])
normalization<-as.integer(args[5])

#Creating plot file
if ( normalization==1 ) rocplot=paste(prefix,'_results/',prefix,'_plots/',prefix,'_plot_roc','.png', sep='')
if ( normalization==2 ) rocplot=paste(prefix,'_results/',prefix,'_plots/',prefix,'_plot_rocn','.png', sep='')
png(filename=rocplot, width=12, height=10, units='cm', res=300)

#Defining margins
par(mar=c(3.2,3.2,0.5,0.5))

#Initializing ROC plot
plot(x=c(-2,2), y=c(-2,2), col='gray', lty=3, lwd=1, xlim=c(-0.05,1.0), ylim=c(0.0,1.02), axes=FALSE, xlab=NA, ylab=NA)
clip(0,1,0,1)
abline(a=0, b=1, col='gray', lty=2, lwd=1)
clip(-1,2,-1,2)

#Initializing vectors
arocb<-vector('numeric',m+1)
aroc<-vector('expression',m+1)
lwidth<-vector('character',m+1)
lltp<-vector('integer',m+1)
lcol<-vector('character',m+1)

#Defining formatting of ROC plot
mval=0
for ( i in 0:m+1 ) {
  if ( mode==2 ) {
    ia=''
    mstring=as.character(i)
    lwidth[i]='0.5'
    lltp[i]=1
    lcol[i]='red'
  }
  else if ( i==m+1 ) {
    ia=''
    lintsp=1
    mstring=''
    lwidth[i]='2'
    lltp[i]=1
    lcol[i]='red'
  }

  #Generating data frames from raster maps
  observ<-paste(temppath,'/observed',mstring,'.asc', sep='')
  prob<-paste(temppath,'/probability',mstring,'.asc', sep='')
  observed<-readGDAL(fname=observ) #observation
  probability<-readGDAL(fname=prob) #probability

  #Preprocessing data frames
  probability$band1[probability$band1<0]<-NaN
  if ( normalization==2 ) probability$band1[probability$band1==0 & observed$band1==0]<-NaN
  observed$band1[observed$band1>0]<-1
  observed$band1[observed$band1<0]<-NaN
  probability$band1[is.na(observed$band1)==TRUE]<-NaN
  observed$band1[is.na(probability$band1)==TRUE]<-NaN

  if ( normalization==2 ) {

    npos<-sum(observed$band1==1, na.rm = TRUE)
    nneg<-sum(observed$band1==0, na.rm = TRUE)
    nadd<-5*npos-nneg

    if ( nadd>=0 ) {
      vectadd<-rep(0,nadd)
      obsroc<-c(observed$band1,vectadd)
      probroc<-c(probability$band1,vectadd)
      ctrlval=1

    } else {
      ctrlval=0
    }
  } else {
    obsroc<-observed$band1
    probroc<-probability$band1
    ctrlval=1
  }

  #Writing averages of observation and model to output file
  if ( mode==1 && normalization==2 && ctrlval==1 ) {
    write(c(paste('AVGobs',as.character(round(mean(obsroc, na.rm=TRUE),4)), sep='\t'),
      paste('AVGmodel',as.character(round(mean(probroc, na.rm=TRUE),4)), sep='\t')),
      file=paste(prefix,'_results/',prefix,'_files/',prefix,'_averages.txt', sep=''), append=FALSE)
  }

  if ( ctrlval==1 ) {

    #Performing ROC analysis
    roc<-prediction(probroc, obsroc) #ROC prediction
    proc<-performance(roc,'tpr', 'fpr') #performance
    arocv<-performance(roc, 'auc') #area under curve

    #Formatting area under curve value for display
    arocb[i]<-as.numeric(arocv@y.values)
    aroc[i]<-substitute(expression(iaa~aroca),
      list(iaa = format(ia), aroca = format(round(as.numeric(arocv@y.values), digits=3), nsmall=3)))[2]

    #Plotting ROC curve
    plot(proc, add=TRUE, axes=F, xlab=NA, ylab=NA, lwd=lwidth[i], col=lcol[i], lty=lltp[i]) #ROC curve
    mval=mval+1

  } else {
    arocb[i]<-NA
    aroc[i]<-NA
  }
}

#Plotting bounding box and axes
box()
axis(side=1, tck=-0.02, labels=NA)
axis(side=2, tck=-0.02, labels=NA)
axis(side=1, lwd=0, line=-0.4)
axis(side=2, lwd=0, line=-0.4)

#Plotting axis labels
mtext(side=1, 'rFP/rON', line=1.9)
mtext(side=2, 'rTP/rOP', line=2)

#Plotting legend
if ( mode==1) {

  aroch<-vector('expression',1)
  aroch[1]<-substitute(expression(AUC[ROC]))[2]
  legend('bottomright', pch=NA, lty=c(0,lltp), lwd=c(0,lwidth), col=c('black',lcol), legend=c(aroch[1],aroc), y.intersp=c(1,lintsp),
      text.col=c('black',lcol), bty='n', adj=c(0.1,0.5))

} else if ( mode==2) {

  arocmean=mean(arocb, na.rm=TRUE)
  arocstdev=sd(arocb, na.rm=TRUE)
  arocmin=min(arocb, na.rm=TRUE)
  arocmax=max(arocb, na.rm=TRUE)

  arocstats<-vector('expression',5)
  arocstats[1]<-substitute(expression(AUC[ROC]~(italic(n) == ma)), list(ma = format(mval)))[2]
  arocstats[2]<-substitute(expression(~italic(mu) == arocmeana), list(arocmeana = format(round(arocmean, digits=3), nsmall=3)))[2]
  arocstats[3]<-substitute(expression(~italic(sigma) == arocstdeva), list(arocstdeva = format(round(arocstdev, digits=3), nsmall=3)))[2]
  arocstats[4]<-substitute(expression(~Min == arocmina), list(arocmina = format(round(arocmin, digits=3), nsmall=3)))[2]
  arocstats[5]<-substitute(expression(~Max == arocmaxa), list(arocmaxa = format(round(arocmax, digits=3), nsmall=3)))[2]

  legend('bottomright', pch=NA, lty=0, lwd=0, col='black', legend=arocstats, y.intersp=c(1.1,0.98,1,1,1),
      text.col=c(lcol[i],'black','black','black','black'), bty='n', adj=c(0.1,0))
}

#Writing area under curve to text file
if ( ctrlval==1 ) {
  if (mode==1 && normalization==1 ) write(paste(prefix, '\t', round(as.numeric(arocv@y.values),3), sep=''), file='aucroc.txt', append=TRUE)
  if (mode==1 && normalization==2 ) write(paste(prefix, '\t', round(as.numeric(arocv@y.values),3), sep=''), file='aucrocn.txt', append=TRUE)
}

#Closing plot file
dev.off()
